@@ -9,6 +9,7 @@ from ..core.anonymization import anonymize_response_data
 from ..core.cache import get_course_code, get_course_id
 from ..core.client import fetch_all_paginated_results, make_canvas_request
 from ..core.dates import format_date, truncate_text
+from ..core.logging import log_error
 from ..core.validation import validate_params
 
 
@@ -190,7 +191,12 @@ def register_assignment_tools(mcp: FastMCP):
         try:
             submissions = anonymize_response_data(submissions, data_type="submissions")
         except Exception as e:
-            print(f"Warning: Failed to anonymize submission data in peer reviews: {str(e)}")
+            log_error(
+                "Failed to anonymize submission data in peer reviews",
+                exc=e,
+                course_id=course_id,
+                assignment_id=assignment_id
+            )
             # Continue with original data for functionality
 
         # Get all users in the course for name lookups
@@ -206,7 +212,12 @@ def register_assignment_tools(mcp: FastMCP):
         try:
             users = anonymize_response_data(users, data_type="users")
         except Exception as e:
-            print(f"Warning: Failed to anonymize user data in peer reviews: {str(e)}")
+            log_error(
+                "Failed to anonymize user data in peer reviews",
+                exc=e,
+                course_id=course_id,
+                assignment_id=assignment_id
+            )
             # Continue with original data for functionality
 
         # Create a mapping of user IDs to names
@@ -311,7 +322,12 @@ def register_assignment_tools(mcp: FastMCP):
         try:
             submissions = anonymize_response_data(submissions, data_type="submissions")
         except Exception as e:
-            print(f"Warning: Failed to anonymize submission data: {str(e)}")
+            log_error(
+                "Failed to anonymize submission data",
+                exc=e,
+                course_id=course_id,
+                assignment_id=assignment_id
+            )
             # Continue with original data for functionality
 
         submissions_info = []
@@ -371,7 +387,12 @@ def register_assignment_tools(mcp: FastMCP):
         try:
             students = anonymize_response_data(students, data_type="users")
         except Exception as e:
-            print(f"Warning: Failed to anonymize student data in analytics: {str(e)}")
+            log_error(
+                "Failed to anonymize student data in analytics",
+                exc=e,
+                course_id=course_id,
+                assignment_id=assignment_id
+            )
             # Continue with original data for functionality
 
         # Get submissions for this assignment
@@ -387,7 +408,12 @@ def register_assignment_tools(mcp: FastMCP):
         try:
             submissions = anonymize_response_data(submissions, data_type="submissions")
         except Exception as e:
-            print(f"Warning: Failed to anonymize submission data in analytics: {str(e)}")
+            log_error(
+                "Failed to anonymize submission data in analytics",
+                exc=e,
+                course_id=course_id,
+                assignment_id=assignment_id
+            )
             # Continue with original data for functionality
 
         # Extract assignment details
